package edu.travel.commodity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.commodity.constant.BaseConstant;
import edu.travel.commodity.dto.ProductSpecDto;
import edu.travel.commodity.entity.*;
import edu.travel.commodity.mapper.ShopProductSpecMapper;
import edu.travel.commodity.service.*;
import edu.travel.commodity.vo.ProductSkuVo;
import edu.travel.commodity.vo.ProductSpecVo;
import edu.travel.commodity.vo.SpecValueVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ShopProductSpecServiceImpl extends ServiceImpl<ShopProductSpecMapper, ShopProductSpec> implements ShopProductSpecService {
    @Autowired
    private ShopProductService shopProductService;
    @Autowired
    private ShopSpecValueService shopSpecValueService;
    @Autowired
    private ShopProductSkuService shopProductSkuService;
    @Autowired
    private ShopSkuSpecValueService shopSkuSpecValueService;
    @Autowired
    private ShopSpecService shopSpecService;

    @Override
    public ProductSpecVo getShopSpec(ProductSpecDto param) {
        LambdaQueryChainWrapper<ShopProduct> query = shopProductService.lambdaQuery().eq(ShopProduct::getId, param.getProductId())
                .eq(ShopProduct::getCountryId, param.getCountryId());
        ShopProduct one = shopProductService.getOne(query);
        if(ObjectUtil.isEmpty(one)){
            return null;
        }
        List<ShopProductSku> list1 = shopProductSkuService.list(Wrappers.<ShopProductSku>lambdaQuery()
                .eq(ShopProductSku::getStatus, BaseConstant.BASIC_STATUS_NO_NUM)
                .eq(ShopProductSku::getProductId, one.getId()));

        if(ObjectUtil.isEmpty(list1)){
            return null;
        }
        ProductSpecVo bean = BeanUtil.toBean(one, ProductSpecVo.class);
        Set<Long> skus = list1.stream().map(ShopProductSku::getSkuId).collect(Collectors.toSet());
        //规格和规格对应的值
        List<ShopSkuSpecValue> skuValues = shopSkuSpecValueService.lambdaQuery().eq(ShopSkuSpecValue::getSkuId, skus).list();
        //规格值ID
        Set<Long> collect = skuValues.stream().map(ShopSkuSpecValue::getSpecValueId).collect(Collectors.toSet());
        List<ShopSpecValue> list2 = shopSpecValueService.list(Wrappers.<ShopSpecValue>lambdaQuery().in(ShopSpecValue::getId, collect));

        //规格ID
        Set<Long> collect3 = list2.stream().map(ShopSpecValue::getSpecId).collect(Collectors.toSet());
        Map<String, ShopSpec> specs = new HashMap<>();
        List<ShopSpec> list = shopSpecService.lambdaQuery().in(ShopSpec::getId, collect3).list();

        for (ShopSpec spec : list) {
            specs.put(spec.getId().toString(), spec);
        }

        Set<Long> skuIds = skuValues.stream().map(ShopSkuSpecValue::getSkuId).collect(Collectors.toSet());
        //SKU -> ShopSpecValue
        ArrayList<ProductSkuVo> productSkuVos = new ArrayList<>();
        for (Long skuId : skuIds) {
            List<Long> collect2 = skuValues.stream().filter(item -> item.getSkuId().equals(skuId)).map(ShopSkuSpecValue::getSpecValueId).collect(Collectors.toList());
            List<ShopSpecValue> collect1 = list2.stream().filter(item -> collect2.contains(item.getId())).collect(Collectors.toList());

            ProductSkuVo productSkuVo = new ProductSkuVo();
            List<SpecValueVo> specValueVos = new ArrayList<>();
            for (ShopSpecValue shopSpecValue : collect1) {
                SpecValueVo specValueVo = new SpecValueVo();
                ShopSpec shopSpec = specs.get(shopSpecValue.getSpecId().toString());
                specValueVo.setKeyId(shopSpecValue.getSpecId().toString());
                specValueVo.setKey(shopSpec.getSpecName());
                specValueVo.setValueId(shopSpecValue.getId().toString());
                specValueVo.setValue(shopSpecValue.getSpecValue());
                specValueVos.add(specValueVo);
            }
            productSkuVo.setSpecValue(specValueVos);
            productSkuVos.add(productSkuVo);
        }
        bean.setSkuList(productSkuVos);

        return bean;
    }
}
